@@ -2,16 +2,16 @@ use proc_macro2::TokenStream;
 use quote::{format_ident, quote};
 use std::collections::BTreeMap;
 use syn::{
-    parse::{Parse, ParseStream},
-    parse_quote, parse_str,
-    punctuated::Punctuated,
-    spanned::Spanned,
-    token::{Async, Comma},
     AngleBracketedGenericArguments, Attribute, Error, Expr, ExprLit, FnArg, GenericArgument, Ident,
     ImplItem, ImplItemFn, ItemImpl,
     Lit::Str,
     Meta, MetaNameValue, PatType, PathArguments, ReturnType, Signature, Token, Type, TypePath,
     Visibility,
+    parse::{Parse, ParseStream},
+    parse_quote, parse_str,
+    punctuated::Punctuated,
+    spanned::Spanned,
+    token::{Async, Comma},
 };
 use zvariant_utils::{case, def_attrs};
 
@@ -25,6 +25,7 @@ def_attrs! {
         name str,
         spawn bool,
         introspection_docs bool,
+        crate_path str,
         proxy {
             // Keep this in sync with proxy's method attributes.
             // TODO: Find a way to share code with proxy module.
@@ -57,6 +58,7 @@ def_attrs! {
                 object str,
                 async_object str,
                 blocking_object str,
+                object_vec none,
                 no_reply none,
                 no_autostart none,
                 allow_interactive_auth none
@@ -73,27 +75,15 @@ def_attrs! {
     };
 }
 
-#[derive(Debug)]
-struct Property<'a> {
+#[derive(Debug, Default)]
+struct Property {
     read: bool,
     write: bool,
     emits_changed_signal: PropertyEmitsChangedSignal,
-    ty: Option<&'a Type>,
+    ty: Option<Type>,
     doc_comments: TokenStream,
 }
 
-impl Property<'_> {
-    fn new() -> Self {
-        Self {
-            read: false,
-            write: false,
-            emits_changed_signal: PropertyEmitsChangedSignal::True,
-            ty: None,
-            doc_comments: quote!(),
-        }
-    }
-}
-
 #[derive(Debug, PartialEq, Copy, Clone)]
 enum MethodType {
     Signal,
@@ -256,11 +246,11 @@ impl MethodInfo {
             let ret = quote!(r);
 
             quote!(match reply {
-                ::std::result::Result::Ok(r) => connection.reply(&hdr, &#ret).await,
-                ::std::result::Result::Err(e) => connection.reply_dbus_error(&hdr, e).await,
+                ::std::result::Result::Ok(r) => __zbus__connection.reply(&hdr, &#ret).await,
+                ::std::result::Result::Err(e) => __zbus__connection.reply_dbus_error(&hdr, e).await,
             })
         } else {
-            quote!(connection.reply(&hdr, &reply).await)
+            quote!(__zbus__connection.reply(&hdr, &reply).await)
         };
 
         let member_name = attrs.name.clone().unwrap_or_else(|| {
@@ -268,6 +258,8 @@ impl MethodInfo {
             if is_property && has_inputs {
                 assert!(name.starts_with("set_"));
                 name = name[4..].to_string();
+            } else if name.starts_with("r#") {
+                name = name[2..].to_string();
             }
             pascal_case(&name)
         });
@@ -297,7 +289,9 @@ impl MethodInfo {
 }
 
 pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Result<TokenStream> {
-    let zbus = zbus_path();
+    let impl_attrs = ImplAttributes::parse_nested_metas(args)?;
+    let crate_path = parse_crate_path(impl_attrs.crate_path.as_deref())?;
+    let zbus = zbus_path(crate_path.as_ref());
 
     let self_ty = &input.self_ty;
     let mut properties = BTreeMap::new();
@@ -324,8 +318,6 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
         }
         _ => return Err(Error::new_spanned(&input.self_ty, "Invalid type")),
     };
-
-    let impl_attrs = ImplAttributes::parse_nested_metas(args)?;
     let iface_name = {
         match (impl_attrs.name, impl_attrs.interface) {
             // Ensure the interface name is valid.
@@ -337,7 +329,7 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                 return Err(syn::Error::new(
                     input.span(),
                     "`name` and `interface` attributes should not be specified at the same time",
-                ))
+                ));
             }
         }
     };
@@ -405,32 +397,31 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
         )?;
         let attr_property = method_attrs.property;
         if let Some(prop_attrs) = &attr_property {
+            let property: &mut Property = properties
+                .entry(method_info.member_name.to_string())
+                .or_default();
             if method_info.method_type == MethodType::Property(PropertyType::Getter) {
                 let emits_changed_signal = if let Some(s) = &prop_attrs.emits_changed_signal {
                     PropertyEmitsChangedSignal::parse(s, method.span())?
                 } else {
                     PropertyEmitsChangedSignal::True
                 };
-                let mut property = Property::new();
+                property.read = true;
                 property.emits_changed_signal = emits_changed_signal;
-                properties.insert(method_info.member_name.to_string(), property);
-            } else if prop_attrs.emits_changed_signal.is_some() {
-                return Err(syn::Error::new(
-                    method.span(),
-                    "`emits_changed_signal` cannot be specified on setters",
-                ));
+            } else {
+                property.write = true;
+                if prop_attrs.emits_changed_signal.is_some() {
+                    return Err(Error::new_spanned(
+                        method,
+                        "`emits_changed_signal` cannot be specified on setters",
+                    ));
+                }
             }
-        };
+        }
         methods.push((method, method_info));
     }
 
     for (method, method_info) in methods {
-        let cfg_attrs: Vec<_> = method
-            .attrs
-            .iter()
-            .filter(|a| a.path().is_ident("cfg"))
-            .collect();
-
         let info = method_info.clone();
         let MethodInfo {
             method_type,
@@ -447,6 +438,7 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
             args_names,
             reply,
             member_name,
+            cfg_attrs,
             ..
         } = method_info;
 
@@ -507,10 +499,7 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                 });
             }
             MethodType::Property(_) => {
-                let p = properties.get_mut(&member_name).ok_or(Error::new_spanned(
-                    &member_name,
-                    "Write-only properties aren't supported yet",
-                ))?;
+                let p = properties.get_mut(&member_name).unwrap();
 
                 let sk_member_name = case::snake_or_kebab_case(&member_name, true);
                 let prop_changed_method_name = format_ident!("{sk_member_name}_changed");
@@ -518,17 +507,11 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
 
                 p.doc_comments.extend(doc_comments);
                 if has_inputs {
-                    p.write = true;
-
                     let set_call = if is_result_output {
                         quote!(self.#ident(#args_names)#method_await)
                     } else if is_async {
                         quote!(
-                                #zbus::export::futures_util::future::FutureExt::map(
-                                    self.#ident(#args_names),
-                                    ::std::result::Result::Ok,
-                                )
-                                .await
+                            ::std::result::Result::Ok(self.#ident(#args_names).await)
                         )
                     } else {
                         quote!(
@@ -555,50 +538,57 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                         }
                     };
 
-                    let value_param = typed_inputs.iter().find(|input| {
-                        let a = ArgAttributes::parse(&input.attrs).unwrap();
-                        !a.object_server
-                            && !a.connection
-                            && !a.header
-                            && !a.signal_context
-                            && !a.signal_emitter
-                    });
-
-                    let value_arg = match &*value_param
-                    .ok_or_else(|| Error::new_spanned(inputs, "Expected a value argument"))?
-                    .ty
-                {
-                    Type::Reference(_) => quote!(value),
-                    Type::Path(path) => path
-                        .path
-                        .segments
-                        .first()
-                        .map(|segment| match &segment.arguments {
-                            PathArguments::AngleBracketed(angled) => angled
-                                .args
-                                .first()
-                                .filter(|arg| matches!(arg, GenericArgument::Lifetime(_)))
-                                .map(|_| quote!(match ::zbus::zvariant::Value::try_clone(value) {
-                                    ::std::result::Result::Ok(val) => val,
-                                    ::std::result::Result::Err(e) => {
-                                        return ::std::result::Result::Err(
-                                            ::std::convert::Into::into(#zbus::Error::Variant(::std::convert::Into::into(e)))
-                                        );
-                                    }
-                                }))
-                                .unwrap_or_else(|| value_to_owned.clone()),
-                            _ => value_to_owned.clone(),
+                    let value_param = typed_inputs
+                        .iter()
+                        .find(|input| {
+                            let a = ArgAttributes::parse(&input.attrs).unwrap();
+                            !a.object_server
+                                && !a.connection
+                                && !a.header
+                                && !a.signal_context
+                                && !a.signal_emitter
                         })
-                        .unwrap_or_else(|| value_to_owned.clone()),
-                    _ => value_to_owned,
-                };
+                        .ok_or_else(|| Error::new_spanned(inputs, "Expected a value argument"))?;
 
-                    let value_param_name = &value_param.unwrap().pat;
+                    // Use setter argument type as the property type if the getter is not
+                    // explicitly defined.
+                    if !p.read {
+                        p.ty = Some((*value_param.ty).clone());
+                        p.emits_changed_signal = PropertyEmitsChangedSignal::False;
+                    }
+
+                    let value_arg = match &*value_param.ty {
+                        Type::Reference(_) => quote!(value),
+                        Type::Path(path) => path
+                            .path
+                            .segments
+                            .first()
+                            .map(|segment| match &segment.arguments {
+                                PathArguments::AngleBracketed(angled) => angled
+                                    .args
+                                    .first()
+                                    .filter(|arg| matches!(arg, GenericArgument::Lifetime(_)))
+                                    .map(|_| quote!(match ::zbus::zvariant::Value::try_clone(value) {
+                                        ::std::result::Result::Ok(val) => val,
+                                        ::std::result::Result::Err(e) => {
+                                            return ::std::result::Result::Err(
+                                                ::std::convert::Into::into(#zbus::Error::Variant(::std::convert::Into::into(e)))
+                                            );
+                                        }
+                                    }))
+                                    .unwrap_or_else(|| value_to_owned.clone()),
+                                _ => value_to_owned.clone(),
+                            })
+                            .unwrap_or_else(|| value_to_owned.clone()),
+                        _ => value_to_owned,
+                    };
+
+                    let value_param_name = &value_param.pat;
                     let prop_changed_method = match p.emits_changed_signal {
                         PropertyEmitsChangedSignal::True => {
                             quote!({
                                 self
-                                    .#prop_changed_method_name(&signal_emitter)
+                                    .#prop_changed_method_name(&__zbus__signal_emitter)
                                     .await
                                     .map(|_| set_result)
                                     .map_err(Into::into)
@@ -607,7 +597,7 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                         PropertyEmitsChangedSignal::Invalidates => {
                             quote!({
                                 self
-                                    .#prop_invalidate_method_name(&signal_emitter)
+                                    .#prop_invalidate_method_name(&__zbus__signal_emitter)
                                     .await
                                     .map(|_| set_result)
                                     .map_err(Into::into)
@@ -664,8 +654,8 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                 } else {
                     let is_fallible_property = is_result_output;
 
-                    p.ty = Some(get_return_type(output)?);
-                    p.read = true;
+                    p.ty = Some(get_return_type(output)?.clone());
+
                     let value_convert = quote!(
                         <#zbus::zvariant::OwnedValue as ::std::convert::TryFrom<_>>::try_from(
                             <#zbus::zvariant::Value as ::std::convert::From<_>>::from(
@@ -693,7 +683,7 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                     get_dispatch.extend(q);
 
                     let q = if is_fallible_property {
-                        quote!(
+                        quote!({
                             #args_from_msg
                             if let Ok(prop) = self.#ident(#args_names)#method_await {
                             props.insert(
@@ -705,19 +695,20 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                                 )
                                 .map_err(|e| #zbus::fdo::Error::Failed(e.to_string()))?,
                             );
-                        })
+                        }})
                     } else {
-                        quote!(
+                        quote!({
                             #args_from_msg
                             props.insert(
-                        ::std::string::ToString::to_string(#member_name),
-                        <#zbus::zvariant::OwnedValue as ::std::convert::TryFrom<_>>::try_from(
-                            <#zbus::zvariant::Value as ::std::convert::From<_>>::from(
-                                self.#ident(#args_names)#method_await,
-                            ),
-                        )
-                        .map_err(|e| #zbus::fdo::Error::Failed(e.to_string()))?,
-                    );)
+                                ::std::string::ToString::to_string(#member_name),
+                                <#zbus::zvariant::OwnedValue as ::std::convert::TryFrom<_>>::try_from(
+                                    <#zbus::zvariant::Value as ::std::convert::From<_>>::from(
+                                        self.#ident(#args_names)#method_await,
+                                    ),
+                                )
+                                .map_err(|e| #zbus::fdo::Error::Failed(e.to_string()))?,
+                            );
+                        })
                     };
 
                     get_all.extend(q);
@@ -729,20 +720,27 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                     };
 
                     if p.emits_changed_signal == PropertyEmitsChangedSignal::True {
+                        let changed_doc = format!(
+                            "Emit the “PropertiesChanged” signal with the new value for the\n\
+                             `{member_name}` property.\n\n\
+                             This method should be called if a property value changes outside\n\
+                             its setter method."
+                        );
                         let prop_changed_method = quote!(
+                            #[doc = #changed_doc]
                             pub async fn #prop_changed_method_name(
                                 &self,
-                                signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
+                                __zbus__signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
                             ) -> #zbus::Result<()> {
-                                let header = ::std::option::Option::None::<&#zbus::message::Header<'_>>;
-                                let connection = signal_emitter.connection();
-                                let object_server = connection.object_server();
+                                let __zbus__header = ::std::option::Option::None::<&#zbus::message::Header<'_>>;
+                                let __zbus__connection = __zbus__signal_emitter.connection();
+                                let __zbus__object_server = __zbus__connection.object_server();
                                 #args_from_msg
                                 let mut changed = ::std::collections::HashMap::new();
                                 let value = <#zbus::zvariant::Value as ::std::convert::From<_>>::from(#prop_value_handled);
                                 changed.insert(#member_name, value);
                                 #zbus::fdo::Properties::properties_changed(
-                                    signal_emitter,
+                                    __zbus__signal_emitter,
                                     #zbus::names::InterfaceName::from_static_str_unchecked(#iface_name),
                                     changed,
                                     ::std::borrow::Cow::Borrowed(&[]),
@@ -754,13 +752,21 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                     }
 
                     if p.emits_changed_signal == PropertyEmitsChangedSignal::Invalidates {
+                        let invalidate_doc = format!(
+                            "Emit the “PropertiesChanged” signal for the `{member_name}` property\n\
+                             without including the new value.\n\n\
+                             It is usually better to call `{prop_changed_method_name}` instead so\n\
+                             that interested peers do not need to fetch the new value separately\n\
+                             (causing excess traffic on the bus)."
+                        );
                         let prop_invalidate_method = quote!(
+                            #[doc = #invalidate_doc]
                             pub async fn #prop_invalidate_method_name(
                                 &self,
-                                signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
+                                __zbus__signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
                             ) -> #zbus::Result<()> {
                                 #zbus::fdo::Properties::properties_changed(
-                                    signal_emitter,
+                                    __zbus__signal_emitter,
                                     #zbus::names::InterfaceName::from_static_str_unchecked(#iface_name),
                                     ::std::collections::HashMap::new(),
                                     ::std::borrow::Cow::Borrowed(&[#member_name]),
@@ -782,8 +788,12 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
                         let future = async move {
                             #args_from_msg
                             let reply = self.#ident(#args_names)#method_await;
-                            let hdr = message.header();
-                            #reply
+                            let hdr = __zbus__message.header();
+                            if hdr.primary().flags().contains(zbus::message::Flags::NoReplyExpected) {
+                                Ok(())
+                            } else {
+                                #reply
+                            }
                         };
                         #zbus::object_server::DispatchResult::Async(::std::boxed::Box::pin(async move {
                             future.await
@@ -852,7 +862,7 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
         }
     };
 
-    let proxy = proxy.map(|proxy| proxy.gen()).transpose()?;
+    let proxy = proxy.map(|proxy| proxy.r#gen()).transpose()?;
     let introspect_format_str = format!("{}<interface name=\"{iface_name}\">", "{:indent$}");
 
     Ok(quote! {
@@ -876,13 +886,13 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
 
             async fn get(
                 &self,
-                property_name: &str,
-                object_server: &#zbus::ObjectServer,
-                connection: &#zbus::Connection,
-                header: Option<&#zbus::message::Header<'_>>,
-                signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
+                __zbus__property_name: &str,
+                __zbus__object_server: &#zbus::ObjectServer,
+                __zbus__connection: &#zbus::Connection,
+                __zbus__header: Option<&#zbus::message::Header<'_>>,
+                __zbus__signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
             ) -> ::std::option::Option<#zbus::fdo::Result<#zbus::zvariant::OwnedValue>> {
-                match property_name {
+                match __zbus__property_name {
                     #get_dispatch
                     _ => ::std::option::Option::None,
                 }
@@ -890,10 +900,10 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
 
             async fn get_all(
                 &self,
-                object_server: &#zbus::ObjectServer,
-                connection: &#zbus::Connection,
-                header: Option<&#zbus::message::Header<'_>>,
-                signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
+                __zbus__object_server: &#zbus::ObjectServer,
+                __zbus__connection: &#zbus::Connection,
+                __zbus__header: Option<&#zbus::message::Header<'_>>,
+                __zbus__signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
             ) -> #zbus::fdo::Result<::std::collections::HashMap<
                 ::std::string::String,
                 #zbus::zvariant::OwnedValue,
@@ -908,14 +918,14 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
 
             fn set<'call>(
                 &'call self,
-                property_name: &'call str,
+                __zbus__property_name: &'call str,
                 value: &'call #zbus::zvariant::Value<'_>,
-                object_server: &'call #zbus::ObjectServer,
-                connection: &'call #zbus::Connection,
-                header: Option<&'call #zbus::message::Header<'_>>,
-                signal_emitter: &'call #zbus::object_server::SignalEmitter<'_>,
+                __zbus__object_server: &'call #zbus::ObjectServer,
+                __zbus__connection: &'call #zbus::Connection,
+                __zbus__header: Option<&'call #zbus::message::Header<'_>>,
+                __zbus__signal_emitter: &'call #zbus::object_server::SignalEmitter<'_>,
             ) -> #zbus::object_server::DispatchResult<'call> {
-                match property_name {
+                match __zbus__property_name {
                     #set_dispatch
                     _ => #zbus::object_server::DispatchResult::NotFound,
                 }
@@ -923,14 +933,14 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
 
             async fn set_mut(
                 &mut self,
-                property_name: &str,
+                __zbus__property_name: &str,
                 value: &#zbus::zvariant::Value<'_>,
-                object_server: &#zbus::ObjectServer,
-                connection: &#zbus::Connection,
-                header: Option<&#zbus::message::Header<'_>>,
-                signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
+                __zbus__object_server: &#zbus::ObjectServer,
+                __zbus__connection: &#zbus::Connection,
+                __zbus__header: Option<&#zbus::message::Header<'_>>,
+                __zbus__signal_emitter: &#zbus::object_server::SignalEmitter<'_>,
             ) -> ::std::option::Option<#zbus::fdo::Result<()>> {
-                match property_name {
+                match __zbus__property_name {
                     #set_mut_dispatch
                     _ => ::std::option::Option::None,
                 }
@@ -938,9 +948,9 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
 
             fn call<'call>(
                 &'call self,
-                object_server: &'call #zbus::ObjectServer,
-                connection: &'call #zbus::Connection,
-                message: &'call #zbus::message::Message,
+                __zbus__object_server: &'call #zbus::ObjectServer,
+                __zbus__connection: &'call #zbus::Connection,
+                __zbus__message: &'call #zbus::message::Message,
                 name: #zbus::names::MemberName<'call>,
             ) -> #zbus::object_server::DispatchResult<'call> {
                 match name.as_str() {
@@ -951,9 +961,9 @@ pub fn expand(args: Punctuated<Meta, Token![,]>, mut input: ItemImpl) -> syn::Re
 
             fn call_mut<'call>(
                 &'call mut self,
-                object_server: &'call #zbus::ObjectServer,
-                connection: &'call #zbus::Connection,
-                message: &'call #zbus::message::Message,
+                __zbus__object_server: &'call #zbus::ObjectServer,
+                __zbus__connection: &'call #zbus::Connection,
+                __zbus__message: &'call #zbus::message::Message,
                 name: #zbus::names::MemberName<'call>,
             ) -> #zbus::object_server::DispatchResult<'call> {
                 match name.as_str() {
@@ -1016,7 +1026,7 @@ fn get_args_from_inputs(
                 }
 
                 let server_arg = &input.pat;
-                server_arg_decl = Some(quote! { let #server_arg = &object_server; });
+                server_arg_decl = Some(quote! { let #server_arg = &__zbus__object_server; });
             } else if connection {
                 if conn_arg_decl.is_some() {
                     return Err(Error::new_spanned(
@@ -1026,7 +1036,7 @@ fn get_args_from_inputs(
                 }
 
                 let conn_arg = &input.pat;
-                conn_arg_decl = Some(quote! { let #conn_arg = &connection; });
+                conn_arg_decl = Some(quote! { let #conn_arg = &__zbus__connection; });
             } else if header {
                 if header_arg_decl.is_some() {
                     return Err(Error::new_spanned(
@@ -1039,9 +1049,12 @@ fn get_args_from_inputs(
 
                 header_arg_decl = match method_type {
                     MethodType::Property(_) => Some(quote! {
-                        let #header_arg = ::std::option::Option::<&#zbus::message::Header<'_>>::cloned(header);
+                        let #header_arg =
+                            ::std::option::Option::<&#zbus::message::Header<'_>>::cloned(
+                                __zbus__header,
+                            );
                     }),
-                    _ => Some(quote! { let #header_arg = message.header(); }),
+                    _ => Some(quote! { let #header_arg = __zbus__message.header(); }),
                 };
             } else if signal_context || signal_emitter {
                 if signal_emitter_arg_decl.is_some() {
@@ -1055,16 +1068,16 @@ fn get_args_from_inputs(
 
                 signal_emitter_arg_decl = match method_type {
                     MethodType::Property(_) => Some(
-                        quote! { let #signal_context_arg = ::std::clone::Clone::clone(signal_emitter); },
+                        quote! { let #signal_context_arg = ::std::clone::Clone::clone(__zbus__signal_emitter); },
                     ),
                     _ => Some(quote! {
                         let #signal_context_arg = match hdr.path() {
                             ::std::option::Option::Some(p) => {
-                                #zbus::object_server::SignalEmitter::new(connection, p).expect("Infallible conversion failed")
+                                #zbus::object_server::SignalEmitter::new(__zbus__connection, p).expect("Infallible conversion failed")
                             }
                             ::std::option::Option::None => {
                                 let err = #zbus::fdo::Error::UnknownObject("Path Required".into());
-                                return connection.reply_dbus_error(&hdr, err).await;
+                                return __zbus__connection.reply_dbus_error(&hdr, err).await;
                             }
                         };
                     }),
@@ -1078,20 +1091,20 @@ fn get_args_from_inputs(
         let (hdr_init, msg_init, args_decl) = match method_type {
             MethodType::Property(PropertyType::Getter) => (quote! {}, quote! {}, quote! {}),
             MethodType::Property(PropertyType::Setter) => (
-                quote! { let hdr = header.as_ref().unwrap(); },
+                quote! { let hdr = __zbus__header.as_ref().unwrap(); },
                 quote! {},
                 quote! {},
             ),
             _ => (
-                quote! { let hdr = message.header(); },
-                quote! { let msg_body = message.body(); },
+                quote! { let hdr = __zbus__message.header(); },
+                quote! { let msg_body = __zbus__message.body(); },
                 quote! {
                     let (#(#args_names),*): (#(#tys),*) =
                         match msg_body.deserialize() {
                             ::std::result::Result::Ok(r) => r,
                             ::std::result::Result::Err(e) => {
                                 let err = <#zbus::fdo::Error as ::std::convert::From<_>>::from(e);
-                                return connection.reply_dbus_error(&hdr, err).await;
+                                return __zbus__connection.reply_dbus_error(&hdr, err).await;
                             }
                         };
                 },
@@ -1168,6 +1181,7 @@ fn introspect_input_args<'i>(
 
             let ident = pat_ident(pat_type).unwrap();
             let arg_name = quote!(#ident).to_string();
+            let arg_name = arg_name.strip_prefix("r#").unwrap_or(arg_name.as_str());
             let dir = if is_signal { "" } else { " direction=\"in\"" };
             let format_str = format!(
                 "{}<arg name=\"{arg_name}\" type=\"{}\"{dir}/>",
@@ -1201,7 +1215,7 @@ fn is_special_arg(attrs: &[Attribute]) -> bool {
             return false;
         };
 
-        let res = nested.iter().any(|nested_meta| {
+        nested.iter().any(|nested_meta| {
             matches!(
                 nested_meta,
                 Meta::Path(path)
@@ -1211,9 +1225,7 @@ fn is_special_arg(attrs: &[Attribute]) -> bool {
                     path.is_ident("signal_context") ||
                     path.is_ident("signal_emitter")
             )
-        });
-
-        res
+        })
     })
 }
 
@@ -1280,8 +1292,14 @@ fn introspect_add_output_args(
         if let Type::Tuple(t) = ty {
             if let Some(arg_names) = arg_names {
                 if t.elems.len() != arg_names.len() {
-                    // Turn into error
-                    panic!("Number of out arg names different from out args specified")
+                    return Err(Error::new_spanned(
+                        ty,
+                        format!(
+                            "out_args specifies {} names but method returns {} values",
+                            arg_names.len(),
+                            t.elems.len()
+                        ),
+                    ));
                 }
             }
             for i in 0..t.elems.len() {
@@ -1289,7 +1307,23 @@ fn introspect_add_output_args(
                 args.extend(introspect_output_arg(&t.elems[i], name, cfg_attrs));
             }
         } else {
-            args.extend(introspect_output_arg(ty, None, cfg_attrs));
+            // Note: The type might still serialize as a DBus struct/tuple if it has a custom
+            // signature (e.g., `#[zvariant(signature = "(...)")]`), but we can't detect this
+            // from the Rust type structure alone.
+            //
+            // If out_args has exactly one name, apply it to this output.
+            // If out_args has multiple names, the user likely has a type with a tuple signature,
+            // but we can't generate separate <arg> elements for each field since we don't have
+            // access to the individual field types. In this case, we generate a single <arg>
+            // with the type's full signature and no name. The signature will be correct at runtime.
+            let name = arg_names.and_then(|names| {
+                if names.len() == 1 {
+                    Some(&names[0])
+                } else {
+                    None
+                }
+            });
+            args.extend(introspect_output_arg(ty, name, cfg_attrs));
         }
     }
 
@@ -1321,7 +1355,7 @@ fn get_return_type(output: &ReturnType) -> syn::Result<&Type> {
 
 fn introspect_properties(
     introspection: &mut TokenStream,
-    properties: BTreeMap<String, Property<'_>>,
+    properties: BTreeMap<String, Property>,
 ) -> syn::Result<()> {
     for (name, prop) in properties {
         let access = if prop.read && prop.write {
@@ -1331,14 +1365,9 @@ fn introspect_properties(
         } else if prop.write {
             "write"
         } else {
-            return Err(Error::new_spanned(
-                name,
-                "property is neither readable nor writable",
-            ));
+            unreachable!("Properties should have at least one access type");
         };
-        let ty = prop.ty.ok_or_else(|| {
-            Error::new_spanned(&name, "Write-only properties aren't supported yet")
-        })?;
+        let ty = prop.ty.unwrap();
 
         let doc_comments = prop.doc_comments;
         if prop.emits_changed_signal == PropertyEmitsChangedSignal::True {
@@ -1453,7 +1482,7 @@ impl Proxy {
     fn add_method(
         &mut self,
         method_info: MethodInfo,
-        properties: &BTreeMap<String, Property<'_>>,
+        properties: &BTreeMap<String, Property>,
     ) -> syn::Result<()> {
         let inputs: Punctuated<PatType, Comma> = method_info
             .typed_inputs
@@ -1504,7 +1533,7 @@ impl Proxy {
         let member_name = method_info.member_name;
         let mut proxy_method_attrs = quote! { name = #member_name, };
         proxy_method_attrs.extend(match method_info.method_type {
-            MethodType::Signal => quote!(signal),
+            MethodType::Signal => quote!(signal,),
             MethodType::Property(_) => {
                 let emits_changed_signal = properties
                     .get(&member_name)
@@ -1513,7 +1542,7 @@ impl Proxy {
                     .to_string();
                 let emits_changed_signal = quote! { emits_changed_signal = #emits_changed_signal };
 
-                quote! { property(#emits_changed_signal) }
+                quote! { property(#emits_changed_signal), }
             }
             MethodType::Other => quote!(),
         });
@@ -1527,6 +1556,9 @@ impl Proxy {
             if let Some(blocking_object) = attrs.blocking_object {
                 proxy_method_attrs.extend(quote! { blocking_object = #blocking_object, });
             }
+            if attrs.object_vec {
+                proxy_method_attrs.extend(quote! { object_vec, });
+            }
             if attrs.no_reply {
                 proxy_method_attrs.extend(quote! { no_reply, });
             }
@@ -1549,7 +1581,7 @@ impl Proxy {
         Ok(())
     }
 
-    fn gen(&self) -> syn::Result<TokenStream> {
+    fn r#gen(&self) -> syn::Result<TokenStream> {
         let attrs = &self.attrs;
         let (
             assume_defaults,
